#!/usr/bin/env python
#####################################################################################
#
# Copyright (c) Members of the EGEE Collaboration. 2004.
# See http://eu-egee.org/partners/ for details on the copyright holders.
# For license conditions see the license file or http://eu-egee.org/license.html
#
#####################################################################################
# glite-dpm-server-config v. 1.5.0
#
# Post-installation script for configuring the gLite dpm servers
# Guillermo Diez-Andino Sancho <Guillermo.Sancho@cern.ch>
#
# Version info: $Id: glite-dpm-server-config.py,v 1.12.2.1 2006/02/11 13:41:05 dimeglio Exp $
#
# Usage: python glite-dpm-server-config [-c|-v|-h|--configure|--start|--stop|--status|--help]
#        -c          print configuration
#        -v          print version
#        --configure configure the services
#        --ldif      update the LCG Providers ldif file and exit
#        --start     start the services
#        --stop      stopt he services
#        --status    check the services status
#        -h,--help   print usage info
#
# Return codes: 0 - Ok
#               1 - Configuration failed
#
#####################################################################################
#The databases are not removed in the configure function if they already exist

# import system libraries
import os,string
import sys
import getopt
import time

from gLiteInstallerLib import gLib
from gLiteInstallerLib import ConfigParams
from  gridSecurity import gridSecurity
from mysql import Mysql
from gliteRgmaServicetool import gliteRgmaServicetoolInstance
from gliteRgmaServicetool import gliteRgmaServicetool

# Set global variables here 
global params         			# all config values from the XML file
global glib

class glite_dpm_server:

	def __init__(self):
		self.verbose = 0
		self.version = "1.5.0"
		self.name = "glite-dpm-server"
		self.friendly_name = "gLite dpm Server"
		if params['db.type']=='mysql':
			self.mysql = Mysql()
		self.commands = {}
		self.commands['dpnsLS'] = "/opt/lcg/bin/dpns-ls"
		self.commands['dpnsMKDIR'] = "/opt/lcg/bin/dpns-mkdir"
		self.commands['dpnsCHMOD'] = "/opt/lcg/bin/dpns-chmod"
		self.commands['dpnsCHOWN'] = "/opt/lcg/bin/dpns-chown"
		self.commands['dpnsACL'] = "/opt/lcg/bin/dpns-setacl -m d:u::7,d:g::7,d:o:5"
		
	#-------------------------------------------------------------------------------
	# Banner 
	#-------------------------------------------------------------------------------
	def banner(self):

		print "\nxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx"
		print "          Configuring the %s" % self.friendly_name
		print "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx\n"
    
	#-------------------------------------------------------------------------------
	# Copyright 
	#-------------------------------------------------------------------------------
	def copyright(self):

		print '\nCopyright (c) Members of the EGEE Collaboration. 2004'
		print 'See http://eu-egee.org/partners/ for details on the copyright holders'
		print 'For license conditions see the license file or http://eu-egee.org/license.html'

	#-------------------------------------------------------------------------------
	# Version
	#-------------------------------------------------------------------------------
	def showVersion(self):

		print '\n%s-config  v. %s\n' % (self.name,self.version)
    
	#-------------------------------------------------------------------------------
	# Usage
	#-------------------------------------------------------------------------------
	def usage(self,msg = ""):

		if msg:
			print "\n%s" % (msg)
        
		self.copyright()
		self.showVersion()
    
		print """Usage: \n
Edit the configuration file %s.cfg.xml in
%s/etc.config/templates\n
save it as %s/etc/config/%s.cfg.xml
and run the script as follows\n 
python %s-config [OPTION...]""" % (self.name, os.environ['GLITE_LOCATION'], \
		os.environ['GLITE_LOCATION'], self.name, self.name)

		print '    -c, --checkconf     print the service configuration'
		print '    -v, --version       print the version of the configuration script'
		print '    -h, --help          print this usage information'
		print '    --configure         configures the dpm services'
		print '    --ldif              update the LCG Providers ldif file and exit'
		print '    --start			   start the services'
		print '    --stop			   stop the services'
		print '    --status            shows the status of the services'
		print '\n'
	
	def backup(self,path):
		pathBak = "%s.1" %path
		if os.path.exists(pathBak):
			os.remove(pathBak)
		if os.path.exists(path):
			os.rename(path,pathBak)
	
	#Creates the namespace and sets the permissions
	def createNameSpace(self):
		volist = glib.getVoList()
		for vo in volist:
			voparams = glib.getVoParams(vo)
			print "Granting VO %s" %voparams['voms.voname']
			path = "dpm/%s/home/%s" %(voparams['vo.domain'],voparams['voms.voname'])
			subPaths = path.split("/")
			absPath = ""

			time.sleep(3)
		
			for relPath in subPaths:
				absPath+="/%s" %relPath
				if os.system("%s %s" %(self.commands['dpnsLS'],absPath)) != 0:
					os.system("%s %s" %(self.commands['dpnsMKDIR'],absPath))
					os.system("%s 775 %s" %(self.commands['dpnsCHMOD'],absPath))
					os.system("%s %s" %(self.commands['dpnsACL'],absPath))
				else:
					print "%s already exists" %absPath,
					glib.printOkMessage()
			#CHOWN
			command = "%s root:%s %s" %(self.commands['dpnsCHOWN'],voparams['pool.account.group'],absPath)
			os.system(command)
		return 0
	
	#Creates the namespace for the IO-servers and the necessary configuration
	def createIOServerNameSpace(self):
		found = 1
		
		instances = glib.getInstances("io-server-dpm")
		for instance in instances:
			print "instance %s" %instance
			iParams = glib.getInstanceParams(instance)
			voName = iParams['io-server.voname']
			host = iParams['io-server.hostname'].split('.')[0]
			
			volist = glib.getVoList()
			for vo in volist:
				voparams = glib.getVoParams(vo)
				if voName == voparams['voms.voname']:
					found = 0
					absPath = "/dpm/%s/home/%s/%s" %(voparams['vo.domain'],voName,host)
					#Opeations to set permissions, etc ....
					print "Setting permissions for io-server-dpm instance %s and path %s" %(instance,absPath)
					if os.system("%s %s" %(self.commands['dpnsLS'],absPath)) != 0:
						os.system("%s %s" %(self.commands['dpnsMKDIR'],absPath))
						os.system("%s 775 %s" %(self.commands['dpnsCHMOD'],absPath))
						os.system("%s %s" %(self.commands['dpnsACL'],absPath))
						command = "%s %s:%s %s" %(self.commands['dpnsCHOWN'],params['dpm.username'],params['dpm.groupname'],absPath)
						os.system(command)
						glib.printOkMessage()
					else:
						print "%s already exists" %absPath,
						glib.printOkMessage()
					
			if(found==1):
				glib.printWarningMessage("The io-server-dpm instance belongs to the vo %s which is not configured, skipping" %voName)
			else:
				found = 0
				
		return 0

	#Creates the tnsnames file
	def createTnsNames(self):
		path = "%s/etc/tnsnames.ora" %os.environ['GLITE_LOCATION']
		params['db.tnsadmin']= "%s/etc" %os.environ['GLITE_LOCATION']
		
		print "Creating tnsnames file under %s" %path	
		self.backup(path)
		file = open(path, 'w')
		file.write("%s=(DESCRIPTION=(ADDRESS=(PROTOCOL=%s)(HOST=%s)(PORT=%s))(CONNECT_DATA=(SID=%s)))\n" \
		%(params['db.name'],params['db.protocol'],params['db.host'],params['db.port'],params['db.sid']))
		file.close()
	
	#Check if the Oracle database already exists	
	def existsOracleSchema(self,serviceDB):
		print "Checking Oracle schema"
		
		path = "check_schema.sql"
		self.backup(path)
		
		if serviceDB=='dpns':
			table = 'CNS_CLASS_METADATA'
		else:
			if serviceDB=='dpm':
				table = 'dpm_pool'
			else:
				table = 'schema_version'	
					
		file = open(path,"w")
		file.write("WHENEVER SQLERROR exit failure;\n")
		file.write("select * from %s;\n" %table)
		file.write("exit success;")
		file.close()
	
		command = "sqlplus -S %s/%s @%s" %(params['db.user'],params['db.password'],path)

		file = os.popen(command, 'r')
		version = file.read()
		if file.close() is None:
			print "There is already a schema loaded into the database",
			glib.printOkMessage()
			return "true"
		else:
			return "false"
			
	def start(self):
		if params['db.type']=='mysql':
			self.mysql.start()
				
		if params['dpns-server.enabled'] == "true":
			service.startService("/opt/lcg/bin/dpnsdaemon","/etc/init.d/dpnsdaemon start")
			time.sleep(3)
			
		
		if params['dpm-server.enabled'] == "true":
			service.startService("/opt/lcg/bin/dpm","/etc/init.d/dpm start")
		
		if params['srmv1-server.enabled'] == "true":
			service.startService("/opt/lcg/bin/srmv1","/etc/init.d/srmv1 start")	
			
		if params['srmv2-server.enabled'] == "true":
			service.startService("/opt/lcg/bin/srmv2","/etc/init.d/srmv2 start")		

		#-------------------------------------------------------------------
		# Start Servicetool
		#-------------------------------------------------------------------
		if params['rgma.servicetool.activate'] == 'true':                                                                                          
			errorcode = rgmaServicetool.start()

			if (errorcode == 0):
				print "The gLite R-GMA Service Publisher has been started           ",                                                                                          
				glib.printOkMessage()
			else:
				glib.printErrorMessage("Could not start the gLite R-GMA Service Publisher")
				glib.printErrorMessage("Please verify and re-run the script")
				return 1
				
		#-------------------------------------------------------------------
		# Start R-GMA gin
		#-------------------------------------------------------------------
		print "Starting the gLite R-GMA GIN service ... "
                       
		errorcode = os.system("%s/etc/config/scripts/glite-rgma-gin-config.py --start --subservice" % os.environ['GLITE_LOCATION'])
		if errorcode:
			print "Starting the gLite R-GMA GIN service                         ",                                                
			glib.printFailedMessage()
			return 1

		print "Starting the gLite R-GMA GIN service                         ",                                                
		glib.printOkMessage()
		return 0

	def stop(self):
		if params['db.type']=='mysql':
			self.mysql.stop()
		
		if params['dpns-server.enabled'] == "true":
			service.stopService("/opt/lcg/bin/dpnsdaemon")
			time.sleep(3)
			
		if params['dpm-server.enabled'] == "true":
			service.stopService("/opt/lcg/bin/dpm")
			
		if params['srmv1-server.enabled'] == "true":
			service.stopService("/opt/lcg/bin/srmv1")
		
		if params['srmv2-server.enabled'] == "true":
			service.stopService("/opt/lcg/bin/srmv2")
			
		#############################################
		#Stop the servicetool
		#############################################
		if params['rgma.servicetool.activate'] == 'true':
			errorcode = rgmaServicetool.stop()
			if (errorcode):
				print "Stopping R-GMA Service Publisher                             ",
				glib.printFailedMessage()
			else:
				print "Stopping R-GMA Service Publisher                             ",
				glib.printOkMessage()
				
		##############################################
        # Stop R-GMA gin
        ##############################################
		print "Stopping the gLite R-GMA GIN service ... "

		errorcode = os.system("%s/etc/config/scripts/glite-rgma-gin-config.py --stop --subservice" % os.environ['GLITE_LOCATION'])
		if errorcode:
			print "Stopping the gLite R-GMA GIN service                         ",
			glib.printFailedMessage()
			return 1

		print "Stopping the gLite R-GMA GIN service                         ",
		glib.printOkMessage()

		return 0
		
	def status(self):
		error_level = 0
		if params['dpns-server.enabled'] == "true":
			if service.statusService("/opt/lcg/bin/dpnsdaemon") != 0:
				error_level = 1
				
		if params['dpm-server.enabled'] == "true":
			if service.statusService("/opt/lcg/bin/dpm") != 0:
				error_level = 1
				
		if params['srmv1-server.enabled'] == "true":
			if service.statusService("/opt/lcg/bin/srmv1") != 0:
				error_level = 1	
		
		if params['srmv2-server.enabled'] == "true":
			if service.statusService("/opt/lcg/bin/srmv2") != 0:
				error_level = 1					

		return error_level
	
	#############################################
	# All the configuration code goes here
	#############################################
	def configure(self):
	
		#############################################
		#Checking Oracle requirements
		#############################################
		if params['db.type'] == 'oracle':
			if params.has_key('dpm.oracle-instantclient.location'):
				if params['dpm.oracle-instantclient.location'] != "":
					if not os.path.exists(params['dpm.oracle-instantclient.location']):
						glib.printErrorMessage("The Oracle InstantClient cannot be found at the specified location")
						glib.printErrorMessage(params['dpm.oracle-instantclient.location'])
						glib.printErrorMessage("Configuration cannot continue")
						return 1
				else:
					glib.printErrorMessage("The Oracle InstantClient location must be defined in dpm.oracle-instantclient.location")
					return 1
			else:
				glib.printErrorMessage("The Oracle InstantClient configuration variable missing: dpm.oracle-instantclient.location")
				return 1
			
			if params['db.tnsadmin'] == "":
				self.createTnsNames()			
				if not os.path.exists("%s/tnsnames.ora" %params['db.tnsadmin']):
					glib.printErrorMessage("The tnsnames.ora file under %s can not be found" %params['db.tnsadmin'])
					return 1
				else:
					print "File tnsnames.ora correctly created under %s" %params['db.tnsadmin'],
					glib.printOkMessage()
		
		#############################################
		#Stopping the services
		#############################################
		print "Stopping the services " 
		if self.stop() != 0:
			return 1
		
		#############################################
		#Configuring the security utililities
		#############################################
		if os.system("python %s/glite-security-utils-config.py --subservice" % glib.getScriptPath()):
			print "\nInstalling gLite Security Utilities               ",
			glib.printFailedMessage()
			return 1
		print "\nInstalling gLite Security Utilities                   ",
		glib.printOkMessage()
		
		#############################################
		#Showing the VOs
		#############################################
		print "\nThis DPM-server will be configured to serve the following VOs:"
		for vo in glib.getVoList():
			print "=> %s" % vo
			
		#############################################
		# Update the $GLITE_LOCATION/etc/glite-mkgridmap.conf file if required
		#############################################
		ret_val = gsec.configure_mkgridmap()
		if ret_val == 1:
			return 1
		elif ret_val == 2:
			error_level = 2
		
		#############################################
		# Create the group map file /etc/grid-security/groupmapfile :
		#############################################
		if gsec.createGroupmapfile():
			return 1
		
		#############################################
		#Checking the host certificates
		#############################################
		if glib.check_certs(params) != 0:
			print "Problems checking the host certificates %s" % service.friendly_name
			sys.exit(1)
		
		#############################################
		#Adding the dpm group
		#############################################
		if self.verbose:
			print "Verifying existence of group %s" % params['dpm.groupname']
		gid = glib.add_group(params['dpm.groupname'],params['dpm.gid'])
		
		if self.verbose:
			print 'Group %s has been added or verified to have GID=%d' % (params['dpm.groupname'], gid)
			
		#############################################
		#Adding the dpm user
		#############################################
		(uid,gid) = glib.add_user(params['dpm.username'],params['dpm.groupname'],params['dpm.uid'])
		if self.verbose:
			print 'User account created                                    ',
			glib.printOkMessage()

		#############################################
		#Creating dpm certificates folder
		#############################################
		dpmCertPath = "/etc/grid-security/%s" %params['dpm.username']
		uid,gid = glib.get_user_info(params['dpm.username'])
		if not os.path.exists(dpmCertPath):
			os.mkdir(dpmCertPath,0755)
			os.chown(dpmCertPath,uid,gid)
		else:
			if self.verbose:
				print "The %s folder already exists" %dpmCertPath,
				glib.printOkMessage()
			
		#############################################
		# Copying the certificates to the dpm user folder
		#############################################
		command = "cp %s %s/dpmcert.pem" %(params['host.certificate.file'],dpmCertPath)
		if self.verbose:
			print "Copying host certificates to %s directory" %params['dpm.username']
			print "Executing %s" %command
		os.system(command)
		os.chown("/etc/grid-security/%s/dpmcert.pem" %params['dpm.username'],uid,gid)
		os.chmod("/etc/grid-security/%s/dpmcert.pem" %params['dpm.username'],0644)
		
		command = "cp %s %s/dpmkey.pem" %(params['host.key.file'],dpmCertPath)
		if self.verbose:
			print "Copying host certificate key to %s directory" %params['dpm.username']
			print "Executing %s" %command
		os.system(command) 
		os.chown("/etc/grid-security/%s/dpmkey.pem" %params['dpm.username'],uid,gid)
		os.chmod("/etc/grid-security/%s/dpmkey.pem" %params['dpm.username'],0400)

		if (os.path.exists('/etc/grid-security/%s/dpmcert.pem' %params['dpm.username']) and os.path.exists('/etc/grid-security/%s/dpmkey.pem' %params['dpm.username']))	:
			print "Copied host certificates and key to /etc/grid-security/%s" %params['dpm.username'],
			glib.printOkMessage()
		else:
			print "An error occurred while copying the host certificates to /etc/grid-security/%s" %params['dpm.username'],
			glib.printFailedMessage()
			return 1
		
		#############################################
		#Checking the grid-mapdir
		#############################################
		if self.verbose:
			print "Checking the gridmapdir"
		if not os.path.exists(params['host.gridmap.dir']):
			os.mkdir(params['host.gridmap.dir'])
			if os.path.exists(params['host.gridmap.dir']):
				print "gridmapdir successfully created",
				glib.printOkMessage(),
			else:
				print "problems to create the gridmapdir under %s" %params['host.gridmap.dir'],
				glib.printFailedMessage()
				return 1
		os.system("chown root:%s %s" %(params['dpm.username'],params['host.gridmap.dir']))

		#############################################
		#Checking the grid-mapfile
		#############################################
		if not os.path.exists(params['host.gridmapfile']):
			print("Creating an empty grid-mapfile under %s" %params['host.gridmapfile'])
			os.system("/bin/touch %s" % params['host.gridmapfile'])
		else:
			print("The grid-mapfile exists under %s" %params['host.gridmapfile']),
			glib.printOkMessage()
			
		#############################################
		# Adding io-servers DN tp the gridmapfile
		#############################################
		instances = glib.getInstances("io-server-dpm")
		for instance in instances:
			iParams = glib.getInstanceParams(instance)
			ioServerDN = iParams['io-server.certificate.subject']
			print "Looking for the host certificate %s in %s" %(ioServerDN,params['host.gridmapfile'])

			found = 0
			file = open(params['host.gridmapfile'],'r')
			for line in file.xreadlines():
				if not line.strip().startswith('#') and line.find("\"%s\"" %ioServerDN) != -1:
					found = 1
					break
			file.close()

			if found == 0:
				file = open(params['host.gridmapfile'],'a')
				file.write("\"%s\" %s\n" %(ioServerDN,params['dpm.username']))
				print("Added host certificate %s to %s" %(ioServerDN,params['host.gridmapfile'])),
				glib.printOkMessage()
				file.close()
			else:
				print("Host certificate with subject %s already exists in %s" %(ioServerDN,params['host.gridmapfile'])),
				glib.printOkMessage()

		#############################################
		#Create pool accounts
		#############################################
		ret_val = gsec.create_pool_accounts()
		if ret_val == 1:
			return 1
		
		#############################################
		#Starting mysql
		#############################################
		#Check backend
		if params['db.type']=='mysql':
			self.mysql.start()
		
		#############################################
		#Configuring root password for mysql
		#############################################
		if params['db.type']=='mysql':
			if not params.has_key('mysql.root.password'):
				params['mysql.root.password'] = ""
			if not params['mysql.root.password'] == "":
				self.mysql.setpassword(params['mysql.root.password'], os.environ['HOSTNAME'])
			
		############################################
		# Configuring the different services (dpns,dpm,srmv1,srmv2)
		############################################
		if params['dpns-server.enabled'] == "true":
			service.configureDPNS()

		if params['dpm-server.enabled'] == "true":
			service.configureDPM()
		
		if params['srmv1-server.enabled'] == "true":
			service.configureSRMV1()
		
		if params['srmv2-server.enabled'] == "true":
			service.configureSRMV2()		

		############################################
		# Configuring the rgma servicetool
		############################################
		if self.configureServiceTool() == 1:
			return 1
                                                                                                                                                               
		print "Configuring the R-GMA Service Publisher                      ",
		glib.printOkMessage()
		
		# Generate the SE LCG Info Provider configuration file
		if params['dpm-server.enabled'] == "true":
			self.generateSEInfoProvidersFile()
		
		############################################
		# R-GMA gin: configure R-GMA gadget-in tool
		############################################
		if self.verbose:
			print "Configuring R-GMA GIn ..."
        
		errorcode = os.system("%s/etc/config/scripts/glite-rgma-gin-config.py --subservice --configure" %os.environ['GLITE_LOCATION'])
		if errorcode:
			print "Configuring R-GMA GadgetIn                                   ",                                                
			glib.printFailedMessage()
			return 1
		else:
			print "Configuring R-GMA GadgetIn                                   ",
			glib.printOkMessage()

		############################################
		# Creating the DPM namespace
		############################################
		if params['dpns-server.enabled'] == "true":
			self.startService("/opt/lcg/bin/dpnsdaemon","/etc/init.d/dpnsdaemon start")
			service.createNameSpace()
			service.createIOServerNameSpace()
			self.stopService("/opt/lcg/bin/dpnsdaemon")
			
		############################################
		# Creating the shift.conf file
		############################################
		if self.authorization() != 0:
			glib.printErrorMessage("Error when creating the /etc/shift.conf file")
			return 1
		else:
			print "/etc/shift.conf file correctly created",
			glib.printOkMessage()	
			
		############################################
		# Stopping mysql
		############################################	
		if params['db.type']=='mysql':
			self.mysql.stop	()
		
		return 0
	
	def configureServiceTool(self):
		serviceTypes = ["lcg.dpns.server","lcg.dpm.server","lcg.dpm.srmv1.server","lcg.dpm.srmv2.server"]
		serviceScripts = ["/etc/init.d/dpnsdaemon status","/etc/init.d/dpm status","/etc/init.d/srmv1 status","/etc/init.d/srmv2 status"]
		
		if params['rgma.servicetool.activate'] == "true":
			for idx in xrange(len(serviceTypes)):
				serviceId = "%s_%s" % (glib.fq_hostname,serviceTypes[idx])
				servicetoolInstance = gliteRgmaServicetoolInstance(glib, serviceId)
				glib.printInfoMessage("Configuring servicetool instance %s" %servicetoolInstance)
                                                                                                                                                               
				# set params
				servicetoolInstance.setServiceName(serviceId)
				servicetoolInstance.setServiceType(serviceTypes[idx])
				servicetoolInstance.setServiceVersion(self.version)
				servicetoolInstance.setStatusScript(serviceScripts[idx])
				servicetoolInstance.setEndpoint('not available')

				# add instance to the gLite configuration
				if servicetoolInstance.add():
					return 1
			
			if rgmaServicetool.configure(glib):
				# error in configuring servicetool
				return 1
		return 0			
		
	def generateSEInfoProvidersFile(self):
		print "Generating SE info provider file"
		
		separamsfile = "%s/etc/lcg-info-generic.conf" % os.environ['GLITE_LOCATION']
		self.backup(separamsfile)
		f = open(separamsfile, 'w')
		f.write("ldif_file=%s/etc/out.ldif\n" % os.environ['GLITE_LOCATION'])
		f.write("generic_script=%s/libexec/lcg-info-generic\n" % params['lcg.providers.location'])
		f.write("wrapper_script=%s/libexec/lcg-info-dynamic-se\n" %params['lcg.providers.location'])
		
		f.write("temp_path=%s\n" %os.environ['GLITE_LOCATION_TMP'])
		f.write("template=%s/etc/GlueSE.template\n" % params['lcg.providers.location'])
		
		f.write("# Common for all\n")
		f.write("GlueInformationServiceURL: undefined\n")
		f.write("\n")
		
		f.write("#DPM SE\n")
		f.write("dn: GlueServiceUniqueID=httpg://%s:8443/srm/managerv1,Mds-Vo-name=local,o=grid\n" %params['srmv1-server.host'])
		f.write("GlueServiceAccessPointURL: httpg://%s:8443/srm/managerv1\n" %params['srmv1-server.host'])
		f.write("GlueServiceEndpoint: httpg://%s:8443/srm/managerv1\n" %params['srmv1-server.host'])
		f.write("GlueServiceType: srm_v1\n")
		f.write("GlueServiceURI: httpg://%s:8443/srm/managerv1\n" %params['srmv1-server.host'])
		f.write("GlueServicePrimaryOwnerName: gLite\n")
		#f.write("GlueServicePrimaryOwnerContact: mailto: %s\n" %params['rgma.site-publisher.contact.system_administrator'])
		f.write("GlueServiceHostingOrganization: %s\n" %params['rgma.servicetool.siteId'])
		f.write("GlueForeignKey: GlueSiteUniqueID=%s\n" %params['rgma.servicetool.siteId'])
		f.write("GlueServiceMajorVersion: 1\n")
		f.write("GlueServiceMinorVersion: 1\n")
		f.write("GlueServicePatchVersion: 1\n")
		
		volist = glib.getVoList()
		for vo in volist:
			voparams = glib.getVoParams(vo)
			f.write("GlueServiceAccessControlRule: %s\n" %voparams['voms.voname'])
		#GlueServiceInformationServiceURL
		f.write("GlueServiceStatus: running\n")
		f.write("\n")
		
		f.write("GlueSEType: dpm\n")
		f.write("GlueSEPort: 2811\n")
		f.write("GlueSESizeTotal: 0\n")
		f.write("GlueSESizeFree: 0\n")
		f.write("GlueSEArchitecture: dpm\n")
		f.write("GlueSAType: permanent\n")
		f.write("GlueSAPolicyFileLifeTime: permanent\n")
		f.write("GlueSAPolicyMaxFileSize: 10000\n")
		f.write("GlueSAPolicyMinFileSize: 1\n")
		f.write("GlueSAPolicyMaxData: 100\n")
		f.write("GlueSAPolicyMaxNumFiles: 10\n")
		f.write("GlueSAPolicyMaxPinDuration: 10\n")
		f.write("GlueSAPolicyQuota: 0\n")
		f.write("GlueSAStateAvailableSpace: 1\n")
		f.write("GlueSAStateUsedSpace: 1\n")
		
		
		f.write("dn: GlueSEUniqueID=%s,mds-vo-name=local,o=grid\n" %os.environ['HOSTNAME'])
		f.write("GlueSEName: %s:dpm\n" %params['rgma.servicetool.siteId'])
		f.write("GlueForeignKey: GlueSiteUniqueID=%s\n" %params['rgma.servicetool.siteId'])
		f.write("\n")                                                                                                                                                 
		f.write("dn: GlueSEAccessProtocolLocalID=gsiftp, GlueSEUniqueID=%s,Mds-Vo-name=local,o=grid\n" %os.environ['HOSTNAME'])
		f.write("GlueSEAccessProtocolType: gsiftp\n")
		f.write("GlueSEAccessProtocolPort: 2811\n")
		f.write("GlueSEAccessProtocolVersion: 1.0.0\n")
		f.write("GlueSEAccessProtocolSupportedSecurity: GSI\n")
		f.write("GlueChunkKey: GlueSEUniqueID=%s\n" %os.environ['HOSTNAME'])
                                                                                                                                                         
		f.write("dn: GlueSEAccessProtocolLocalID=rfio, GlueSEUniqueID=%s,Mds-Vo-name=local,o=grid\n" %os.environ['HOSTNAME'])
		f.write("GlueSEAccessProtocolType: rfio\n")
		f.write("GlueSEAccessProtocolPort: 5001\n")
		f.write("GlueSEAccessProtocolVersion: 1.0.0\n")
		f.write("GlueSEAccessProtocolSupportedSecurity: RFIO\n")
		f.write("GlueChunkKey: GlueSEUniqueID=%s\n" %os.environ['HOSTNAME'])
		
		for vo in volist:
			voparams = glib.getVoParams(vo)
			f.write("dn: GlueSALocalID=%s,GlueSEUniqueID=%s,Mds-Vo-name=local,o=grid\n" %(voparams['voms.voname'],os.environ['HOSTNAME']))
			storageRoot="%s:/dpm/%s/home/%s" %(voparams['voms.voname'],voparams['vo.domain'],voparams['voms.voname'])
			f.write("GlueSARoot: %s\n" %storageRoot)
			f.write("GlueSAPath: %s\n" %storageRoot)
			f.write("GlueSAAccessControlBaseRule: %s\n" %vo)
			f.write("GlueChunkKey: GlueSEUniqueID=%s\n" %os.environ['HOSTNAME'])
		f.close()	
		
		# Load configuration in LCG Info Providers
		os.system("%s/sbin/lcg-info-generic-config %s/etc/lcg-info-generic.conf" \
			% (params['lcg.providers.location'],os.environ['GLITE_LOCATION']))


		f = open("%s/libexec/lcg-info-wrapper" %params['lcg.providers.location'], 'w')
		f.write("#!/bin/bash\n")
		f.write("%s/libexec/lcg-info-generic %s" %(params['lcg.providers.location'],separamsfile))
		f.close()

		os.chmod("%s/libexec/lcg-info-wrapper" %params['lcg.providers.location'], 0755)
		os.chmod("%s/etc/out.ldif" % os.environ['GLITE_LOCATION'], 0644)
		
		print "Configuring DPM SE information provider                          ",
		glib.printOkMessage()
				
	def authorization(self):
		############################################
		# Creating the shift.conf file
		############################################
		path = "/etc/shift.conf"
		
		#DPM authorization
		if (params['dpm-server.enabled']=="true"): #It is always needed to add the disk servers when they run in a different machine
			self.backup(path) 			
			dpmTrust = "DPM TRUST";
						
			if len(params['disk-server.list']) <= 0:
				glib.printErrorMessage("It is necessary to specify at least one disk server")
				return 1
				
			for idx in xrange(len(params['disk-server.list'])):
				hostName = params['disk-server.list'][idx].split(".")[0]
				if (params['disk-server.list'][idx] != params['srmv1-server.host']) and (params['disk-server.list'][idx] != params['srmv2-server.host']):
					dpmTrust += " %s %s" %(hostName,params['disk-server.list'][idx])
						
			if params['dpm-server.host'] != params['srmv1-server.host']:
				hostName = params['srmv1-server.host'].split(".")[0]
				dpmTrust += " %s %s" %(hostName,params['srmv1-server.host'])
			
			if (params['dpm-server.host'] != params['srmv2-server.host']) and (params['srmv1-server.host'] != params['srmv2-server.host']):
				hostName = params['srmv2-server.host'].split(".")[0]
				dpmTrust += " %s %s" %(hostName,params['srmv2-server.host'])
				
		#DPNS authorization
		if (params['dpns-server.enabled']=="true"):
			dpnsTrust = "DPNS TRUST"
			
			if len(params['disk-server.list']) <= 0:
				glib.printErrorMessage("It is necessary to specify at least one disk server")
				return 1
				
			for idx in xrange(len(params['disk-server.list'])):
				hostName = params['disk-server.list'][idx].split(".")[0]
				if (params['disk-server.list'][idx] != params['srmv1-server.host']) and (params['disk-server.list'][idx] != params['srmv2-server.host']):
					dpnsTrust += " %s %s" %(hostName,params['disk-server.list'][idx])

			if params['dpns-server.host'] != params['srmv1-server.host']:			
				hostName = params['srmv1-server.host'].split(".")[0]
				dpnsTrust += " %s %s" %(hostName,params['srmv1-server.host'])
			
			if params['dpns-server.host'] != params['srmv2-server.host'] and (params['srmv1-server.host'] != params['srmv2-server.host']):
				hostName = params['srmv2-server.host'].split(".")[0]
				dpnsTrust += " %s %s" %(hostName,params['srmv2-server.host'])
				
			if (dpnsTrust != "DPNS TRUST"):
				dpnsTrust += "\n"		
				file = open(path,"a")
				file.write(dpnsTrust)
				file.close()
			
		#RFIOD authorization
		if (params['dpm-server.enabled']=="true"):
			hostList = ""
			
			#Authorising io-server instances
			instances = glib.getInstances("io-server-dpm")
			for instance in instances:
				iParams = glib.getInstanceParams(instance)
				hostName = iParams['io-server.hostname']
				hostList += " %s %s" %(hostName,iParams['io-server.hostname'])
				
			for idx in xrange(len(params['disk-server.list'])):
				hostName = params['dpm-server.host'].split(".")[0]
				if (params['dpm-server.host'] == params['disk-server.list'][idx]):
					hostList += " %s %s" %(hostName,params['dpm-server.host'])
					break

			if (hostList != ""):
				file = open(path,"a")
				file.write("RFIOD TRUST %s\n" %hostList)
				file.write("RFIOD RTRUST %s\n" %hostList)
				file.write("RFIOD WTRUST %s\n" %hostList)
				file.write("RFIOD XTRUST %s\n" %hostList)
				file.write("RFIOD FTRUST %s\n" %hostList)
				file.write("RFIO DAEMONV3_WRMT 1\n")
				file.close()

		return 0
		
	def startService(self,daemon,startupScript):
		returnCode = 0
		
		self.stopService(daemon)
		os.system(startupScript)
		
		if glib.getPID(daemon) == 0:
			glib.printErrorMessage("The %s daemon has not been properly started" %daemon)
			returnCode = 1
				
		return returnCode	
		
	def stopService(self,daemon):
		returnCode = 0
		
		if glib.getPID(daemon) != 0:
			glib.stopProcess(daemon)
			if glib.getPID(daemon) != 0:
				glib.printErrorMessage("The process %s has not been properly stopped" %daemon)
				returnCode = 1
		else:
			print("The process %s is already stopped" %daemon),
			glib.printOkMessage()
		return returnCode	
	
	def statusService(self,daemon):
		error_level = 0

		pid = glib.getPID(daemon)
		if pid != 0:
			print "The process %s is running" %daemon
		else:
			print "The process %s is not running" %daemon
			error_level = 1
		return error_level	
		
	#############################################
	# Configuring the DPNS service
	#############################################
	def configureDPNS(self):
		print "Configuring the DPNS server"
		path = "/etc/sysconfig/dpnsdaemon"
		dpnsDbName = "cns_db"
		uid,gid = glib.get_user_info(params['dpm.username'])
			
		self.backup(path)

		print "Creating the %s configuration file" %path
		file = open(path, 'w')
		file.write("RUN_DPNSDAEMON=\"yes\"\n")
		file.write("DPNSDAEMONLOGFILE=\"%s\"\n" % params['dpns-server.logfile'])
		file.write("NSCONFIGFILE=\"%s\"\n" % params['dpns-server.configfile'])
		#file.write("DPMUSER=%s\n" %params['dpm.username'])
        #file.write("DPMGROUP=%s\n" %params['dpm.groupname'])
		file.write("export GRIDMAPDIR=\"%s\"\n" % params['host.gridmap.dir'])
		file.write("export GRIDMAP=\"%s\"\n" % params['host.gridmapfile'])
		if params['db.type']=='oracle':
			file.write("export ORACLE_HOME=\"%s\"\n" % params['dpm.oracle-instantclient.location'])
			file.write("export TNS_ADMIN=\"%s\"\n" % params['db.tnsadmin'])
			file.write("export TWO_TASK=\"%s\"\n" % params['db.name'])
			
		file.close()
		print("%s configuration file created " % path),
		glib.printOkMessage()
		#os.chown("/etc/grid-security/dpmmgr/dpmkey.pem",uid,guid)
		
		#Assigning permissions to the log directory
		if not os.path.exists("%s" %params['dpns-server.logfile']):
			os.system("touch %s" %params['dpns-server.logfile'])
		os.system("chown -R %s:%s %s" %(params['dpm.username'],params['dpm.groupname'],params['dpns-server.logfile']))
		
		path = params['dpns-server.configfile']
		self.backup(path)
		
		print "Creating the %s configuration file" %path
		file = open(path, 'w')
		if params['db.type'] == 'oracle':
			file.write("%s/%s@%s" %(params['db.user'],params['db.password'],params['db.name']))
		else:
			file.write("%s/%s@%s" %(params['db.user'],params['db.password'],params['db.host']))
		file.close()

		#Assigning owner
		os.chown(path,uid,gid)
		print("%s configuration file created " % path),
		glib.printOkMessage()
		
		#DATABASE creation
		if params['db.type']=='mysql':
			if self.mysql.existsDB(dpnsDbName,params['mysql.root.password']) != 0:
				os.system("mysql -u root %s < /opt/lcg/share/DPM/create_dpns_tables_mysql.sql" %params["mysql.root.password"])
				if self.mysql.existsDB(dpnsDbName,params['mysql.root.password']) != 0:
					glib.printErrorMessage("Problems to create the database %s") %dpnsDbName
					return 1
				else:
					print("Database %s successfully created" %dpnsDbName),
					glib.printOkMessage()
			else:
				print("The database %s already exists" %dpnsDbName),
				glib.printOkMessage()
			
			#DB USER creation
			self.mysql.add_user_old_format(dpnsDbName,params['db.user'],params['db.password'],params['mysql.root.password'])
		else:
			if params['db.type'] == 'oracle':
				if self.existsOracleSchema('dpns') == "false":
					command = "sqlplus %s/%s < /opt/lcg/share/DPM/create_dpns_tables_oracle.sql" %(params['db.user'],params['db.password'])
					print "Executing: %s" %command
					os.system(command);	
			
		return 0
		
	#############################################
	# Configuring the DPM service
	#############################################
	def configureDPM(self):
		print "Configuring the DPM server"
		path = "/etc/sysconfig/dpm"
		dpmDbName = "dpm_db"
		uid,gid = glib.get_user_info(params['dpm.username'])
			
		self.backup(path)

		print "Creating the %s configuration file" %path
		file = open(path, 'w')
		file.write("RUN_DPMDAEMON=\"yes\"\n")
		#file.write("DPMUSER=%s\n" %params['dpm.username'])
		#file.write("DPMGROUP=%s\n" %params['dpm.groupname'])
		file.write("export DPNS_HOST=%s\n" %params['dpns-server.host'])
		file.write("DPMDAEMONLOGFILE=\"%s\"\n" % params['dpm-server.logfile'])
		file.write("DPMCONFIGFILE=\"%s\"\n" % params['dpm-server.configfile'])
		file.write("export GRIDMAPDIR=\"%s\"\n" % params['host.gridmap.dir'])
		file.write("export GRIDMAP=\"%s\"\n" % params['host.gridmapfile'])
		if params['db.type']=='oracle':
			file.write("export ORACLE_HOME=\"%s\"\n" % params['dpm.oracle-instantclient.location'])
			file.write("export TNS_ADMIN=\"%s\"\n" % params['db.tnsadmin'])
			file.write("export TWO_TASK=\"%s\"\n" % params['db.name'])
		file.close()
		print("%s configuration file created " % path),
		glib.printOkMessage()
		
		#Assigning permissions to the log directory
		if not os.path.exists("%s" %params['dpm-server.logfile']):
			os.system("touch %s" %params['dpm-server.logfile'])
		os.system("chown -R %s:%s %s" %(params['dpm.username'],params['dpm.groupname'],params['dpm-server.logfile']))
		
		path = params['dpm-server.configfile']
		self.backup(path)
		
		print "Creating the %s configuration file" %path
		file = open(path, 'w')
		if params['db.type'] == 'oracle':
			file.write("%s/%s@%s" %(params['db.user'],params['db.password'],params['db.name']))
		else:
			file.write("%s/%s@%s" %(params['db.user'],params['db.password'],params['db.host']))
		file.close()
		#Assigning owner
		os.chown(path,uid,gid)
		print("%s configuration file created " % path),
		glib.printOkMessage()
		
		#DATABASE creation
		if params['db.type']=='mysql':
			if self.mysql.existsDB(dpmDbName,params['mysql.root.password']) != 0:
				os.system("mysql -u root %s < /opt/lcg/share/DPM/create_dpm_tables_mysql.sql" %params["mysql.root.password"])
				if self.mysql.existsDB(dpmDbName,params['mysql.root.password']) != 0:
					glib.printErrorMessage("Problems to create the database %s") %dpmDbName
					return 1
				else:
					print("Database %s successfully created" %dpmDbName),
					glib.printOkMessage()
			else:
				print("The database %s already exists" %dpmDbName),
				glib.printOkMessage()
			
			#DB USER creation
			self.mysql.add_user_old_format(dpmDbName,params['db.user'],params['db.password'],params['mysql.root.password'])
		else:
			if params['db.type'] == 'oracle':
				print "DPM DB check---------------->"
				if self.existsOracleSchema('dpm') == "false":
					command = "sqlplus %s/%s < /opt/lcg/share/DPM/create_dpm_tables_oracle.sql" %(params['db.user'],params['db.password'])
					os.system(command);
			
		return 0
	
	#############################################
	# Configuring the SRMV1 service
	#############################################	
	def configureSRMV1(self):
		print "Configuring the DPM server"
		path = "/etc/sysconfig/srmv1"
		uid,gid = glib.get_user_info(params['dpm.username'])
		
		self.backup(path)
			
		print "Creating the %s configuration file" %path
		file = open(path, 'w')
		file.write("RUN_SRMV1DAEMON=\"yes\"\n")
		#file.write("DPMUSER=%s\n" %params['dpm.username'])
		#file.write("DPMGROUP=%s\n" %params['dpm.groupname'])
		file.write("export DPM_HOST=%s\n" %params['dpm-server.host'])
		file.write("export DPNS_HOST=%s\n" %params['dpns-server.host'])
		file.write("SRMV1DAEMONLOGFILE=\"%s\"\n" % params['srmv1-server.logfile'])
		file.write("DPMCONFIGFILE=\"%s\"\n" % params['dpm-server.configfile'])
		file.write("export GRIDMAPDIR=\"%s\"\n" % params['host.gridmap.dir'])
		file.write("export GRIDMAP=\"%s\"\n" % params['host.gridmapfile'])
		if params['db.type']=='oracle':
			file.write("export ORACLE_HOME=\"%s\"\n" % params['dpm.oracle-instantclient.location'])
			file.write("export TNS_ADMIN=\"%s\"\n" % params['db.tnsadmin'])
			file.write("export TWO_TASK=\"%s\"\n" % params['db.name'])
		file.close()
		print("%s configuration file created " % path),
		glib.printOkMessage()
		
		if params['dpm-server.enabled'] != "true":
			path = params['dpm-server.configfile']
			self.backup(path)
		
			print "Creating the %s configuration file" %path
			file = open(path, 'w')
			if params['db.type'] == 'oracle':
				file.write("%s/%s@%s" %(params['db.user'],params['db.password'],params['db.name']))
			else:
				file.write("%s/%s@%s" %(params['db.user'],params['db.password'],params['db.host']))
			file.close()
			
			#Assigning owner
			os.chown(path,uid,gid)
			print("%s configuration file created " % path),
			glib.printOkMessage()
			
		#Assigning permissions to the log directory
		if not os.path.exists("%s" %params['srmv1-server.logfile']):
			os.system("touch %s" %params['srmv1-server.logfile'])
		os.system("chown -R %s:%s %s" %(params['dpm.username'],params['dpm.groupname'],params['srmv1-server.logfile']))
		#Check that permissions are also assigned to the srmv1 log location

		return 0
		
	#############################################
	# Configuring the SRMV2 service
	#############################################		
	def configureSRMV2(self):
		path = "/etc/sysconfig/srmv2"
		uid,gid = glib.get_user_info(params['dpm.username'])
		
		self.backup(path)
			
		file = open(path,'w')
		file.write("RUN_SRMV2DAEMON=\"yes\"\n")
		#file.write("DPMUSER=%s\n" %params['dpm.username'])
        #file.write("DPMGROUP=%s\n" %params['dpm.groupname'])
		file.write("export DPM_HOST=%s\n" %params['dpm-server.host'])
		file.write("export DPNS_HOST=%s\n" %params['dpns-server.host'])
		file.write("SRMV2DAEMONLOGFILE=\"%s\"\n" % params['srmv2-server.logfile'])
		file.write("DPMCONFIGFILE=\"%s\"\n" % params['dpm-server.configfile'])
		file.write("export GRIDMAPDIR=\"%s\"\n" % params['host.gridmap.dir'])
		file.write("export GRIDMAP=\"%s\"\n" % params['host.gridmapfile'])
		if params['db.type']=='oracle':
			file.write("export ORACLE_HOME=\"%s\"\n" % params['dpm.oracle-instantclient.location'])
			file.write("export TNS_ADMIN=\"%s\"\n" % params['db.tnsadmin'])
			file.write("export TWO_TASK=\"%s\"\n" % params['db.name'])
		file.close()
		print("%s configuration file created " % path),
		glib.printOkMessage()
		
		if params['dpm-server.enabled'] != "true":
			path = params['dpm-server.configfile']
			self.backup(path)
		
			print "Creating the %s configuration file" %path
			file = open(path, 'w')
			if params['db.type'] == 'oracle':
				file.write("%s/%s@%s" %(params['db.user'],params['db.password'],params['db.name']))
			else:
				file.write("%s/%s@%s" %(params['db.user'],params['db.password'],params['db.host']))
			file.close()
			#Assigning owner
			os.chown(path,uid,gid)
			print("%s configuration file created " %path),
			glib.printOkMessage()
			
		#Assigning permissions to the log directory
		if not os.path.exists("%s" %params['srmv2-server.logfile']):
			os.system("touch %s" %params['srmv2-server.logfile'])
		os.system("chown -R %s:%s %s" %(params['dpm.username'],params['dpm.groupname'],params['srmv2-server.logfile']))
		#Check that permissions are also assigned to the srmv2 log location

		return 0		
	
#-------------------------------------------------------------------------------
# Set all environment variables
#-------------------------------------------------------------------------------
def set_env():
	
	glib.export('GLITE_LOCATION');
	glib.export('GLITE_LOCATION_VAR');
	if not os.path.exists(os.environ['GLITE_LOCATION_VAR']):
		os.mkdir(os.environ['GLITE_LOCATION_VAR'],0755)
	glib.export('GLITE_LOCATION_LOG');
	if not os.path.exists(os.environ['GLITE_LOCATION_LOG']):
		os.mkdir(os.environ['GLITE_LOCATION_LOG'],0755)
	glib.export('GLITE_LOCATION_TMP');
	
	if not os.path.exists(os.environ['GLITE_LOCATION_TMP']):
		os.mkdir(os.environ['GLITE_LOCATION_TMP'],0755)

	glib.export('GLOBUS_LOCATION')
	glib.export("DPM_HOST","%s" %params['dpm-server.host'])
	glib.export("DPNS_HOST","%s" %params['dpns-server.host'])	
	
	# bin and lib paths
	glib.addEnvPath("PATH","%s/bin:%s/sbin,%s/libexec:/opt/lcg/bin" % (os.environ['GLITE_LOCATION'],os.environ['GLITE_LOCATION'],os.environ['GLITE_LOCATION']))
	glib.addEnvPath("LD_LIBRARY_PATH","%s/lib:%s/lib:/opt/gpt/lib:/opt/lcg/lib" % (os.environ['GLOBUS_LOCATION'],os.environ['GLITE_LOCATION']))
	
	if params['db.type'] == 'oracle':
		glib.addEnvPath("LD_LIBRARY_PATH","%s/lib" %params['dpm.oracle-instantclient.location'])
		glib.export("TWO_TASK","%s" %params['db.name'])
		if params['db.tnsadmin']=="":
			glib.export("TNS_ADMIN","%s/etc" %os.environ['GLITE_LOCATION'])
		else:
			glib.export("TNS_ADMIN","%s" %params['db.tnsadmin'])
	
	# Set environment
	glib.setUserEnv()


#-------------------------------------------------------------------------------
#  Main program begins here 
#-------------------------------------------------------------------------------

if __name__ == '__main__':
	# The script must be run as root
	if not os.geteuid()==0:
		glib.printFailedMessage("This script must be run as root")
		sys.exit(1)
	
	# Get an instance of the library class
	glib = gLib()
	
	# Load parameters
	params = ConfigParams() 
	glib.loadConfiguration(["%s/../glite-dpm-server.cfg.xml" % glib.getScriptPath(), "%s/../glite-rgma-servicetool.cfg.xml" % glib.getScriptPath()], params)

	try:
		opts, args = glib.getopt(sys.argv[1:], '', ['siteconfig='])
		for o, a in opts:
			if o == "--siteconfig":
				params['site.config.url'] = a
				break
	except getopt.GetoptError:
		pass
    
	# set verbosity
	if params.has_key('glite.installer.verbose'):
		if (params['glite.installer.verbose'] == "true"):
			verbose=1
		else:
			verbose=0
	else:
		verbose = 0
	glib.verbose = verbose
	
	# Set up the environment
	set_env()
	service = glite_dpm_server()
	
	#Instantiate grdSecurity classes
	gsec = gridSecurity(glib)
	gsec.setVerbosity(verbose)
	gsec.params = params

	# Instantiate the rgma servicetool class
	if not params.has_key('rgma.servicetool.activate'):
		params['rgma.servicetool.activate'] = 'true'
	if params['rgma.servicetool.activate'] == 'true':
		rgmaServicetool = gliteRgmaServicetool()
		rgmaServicetool.verbose = verbose
                                                                                                  
	path = '%s/etc' % os.environ['GLITE_LOCATION']
	if not os.path.exists(path):
		glib.printErrorMessage("%s/etc does not exist! Please verify your gLite installation") % os.environ['GLITE_LOCATION']
		sys.exit (1)
	else:
		print '%s/etc exists' % os.environ['GLITE_LOCATION'],
		glib.printOkMessage()

	#Checking that at least one service is enabled
	if params['dpns-server.enabled'] != "true" and params['dpm-server.enabled'] != "true" and params['srmv1-server.enabled'] != "true" and params['srmv2-server.enabled'] != "true":
		glib.printErrorMessage("At least one of either the dpns-server,the dpm-server, the srmv1-server or srmv2-server should be enabled in the configuration file ")
		sys.exit(1)

	# Command line opts if any
	try:
		opts, args = glib.getopt(sys.argv[1:], 'chv', ['checkconf','help','ldif','version','stop','start','status','configure','siteconfig='])
	except getopt.GetoptError:
		service.usage(msg = "[FAILED] Unknown options(s)")
		sys.exit(1)
		
	# Check cli options
	for o, a in opts:
		if o in ("-h", "--help"):
			service.usage()
			sys.exit(0)
		if o in ("-v", "--version"):
			service.showVersion()
			sys.exit(0)
		if o in ("-c", "--checkconf"):
			service.copyright()
			service.showVersion()
			glib.print_params(params)
			sys.exit(0)
		if o == "--start":
			if service.start() == 0:
				print "The %s was successfully started" % service.friendly_name,
				glib.printOkMessage()
				sys.exit(0)
			else:
				glib.printErrorMessage("An error occurred while starting the %s service" % service.friendly_name)
				sys.exit(1)
		if o == "--stop":
			if service.stop() == 0:
				print("The service %s was successfully stopped" % service.friendly_name),
				glib.printOkMessage()
				sys.exit(0)
			else:
				glib.printErrorMessage("An error occurred while stopping the %s service" % service.friendly_name)
				sys.exit(1)
		if o == "--status":
			sys.exit(service.status())
		if o == "--configure":
                        print "start configure"
			if service.configure() == 0:				
				print("%s configuration successfully completed " % service.friendly_name),
				glib.printOkMessage()
				glib.registerService()
				sys.exit(0)
			else:
				glib.printErrorMessage("An error occurred while configuring the %s service" % service.friendly_name)
				sys.exit(1)
		if o == "--ldif":
			service.copyright()
			service.showVersion()
			sys.exit(service.generateSEInfoProvidersFile())		
 
	# Print configuration parameters
	if verbose:
		glib.print_params(params)
		service.copyright()
		service.showVersion()
		service.banner()
	service.usage()	
